#include "TCanvas.h"
#include "TColor.h"
#include "TFile.h"
#include "TH1.h"
#include "TLeaf.h"
#include "TLorentzVector.h"
#include "TPad.h"
#include "TPostScript.h"
#include "TROOT.h"
#include "TSystem.h"
#include "TStyle.h"
#include "TTree.h"

#include "iostream.h"
using std::cout;
using std::endl;

int ntuple_analysis()
{
  // David Waters, 19.06.2001.
  // =========================
  // This macro is intended to analyse the ntuple produced by mcfm. In general
  // it is an ntuple of weighted events, so histograms need to be filled by
  // hand, applying the appropriate weights.

  // Load the Phyics library, containing for example TLorentzVector :
  // ================================================================
  // gSystem->Load("$ROOTSYS/lib/libPhysics.so");

  cout << " MCFM Ntuple Analysis " << endl;
  
  // Clean up :
  // ==========
  gROOT->Reset();
  gStyle->SetFillColor(0);
  gROOT->SetStyle("Plain");
  
  // Make an output file to store the histograms :
  // =============================================
  TFile* hfile = (TFile*) gROOT->FindObject("mcfm_histos.root"); if (hfile) hfile->Close();
  hfile = new TFile("mcfm_histos.root","RECREATE","MCFM Histograms");

  // Make a canvas, postscipt object and pads :
  // ==========================================
  TCanvas* c1 = new TCanvas("c1","MCFM",0,0,600,700);
  // c1->SetCanvasSize(750.,900.);
  c1->Size(0.,10.);

  TPostScript *ps = new TPostScript("mcfm.ps",111);
  ps->Range(18.5,26.0);  
  ps->NewPage();

  // Specify number of plots in x and y here :
  int num_plots_x = 2;
  int num_plots_y = 3;
  int total_pads = num_plots_x*num_plots_y;
  TPad** const padPointers = new TPad*[total_pads];
  for (int i = 0; i < num_plots_x; ++i)
    {
      for (int j = 0; j < num_plots_y; ++j)
	{
	  int xindex = i+1;
	  int yindex = j+1;
	  int padIndex = num_plots_x * (yindex-1) + xindex - 1;
	  // cout << "padIndex = " << padIndex << endl;
	  padPointers[padIndex] = new TPad("pad1"," ",
					   ((1.0*i)/(1.0*num_plots_x)),
					   (((1.0*num_plots_y-1.0)/(1.0*num_plots_y))-((1.0*j)/(1.0*num_plots_y))),
					   ((1.0*i+1.0)/(1.0*num_plots_x)),
					   (1.0-((1.0*j)/(1.0*num_plots_y))),
					   21);

	  // cout << "lower-left x = " << ((1.0*i)/(1.0*num_plots_x)) << endl;
	  // cout << "lower-left y = " << (((1.0*num_plots_y-1.0)/(1.0*num_plots_y))-((1.0*j)/(1.0*num_plots_y))) << endl;
	  // cout << "upper-right x = " << ((1.0*i+1.0)/(1.0*num_plots_x)) << endl;
	  // cout << "upper-right y = " << (1.0-((1.0*j)/(1.0*num_plots_y))) << endl;

	  padPointers[padIndex]->SetFillColor(0);
	  padPointers[padIndex]->SetGrid();
	  // Get rid of irritating red line at the border :
	  padPointers[padIndex]->SetBorderMode(0.);
	  padPointers[padIndex]->Draw();
	}
    }

  // --------------------------------------------------------------------------
  // Open the file containing the merged ntuple and grab the ntuple :
  // ================================================================
  TFile* mcfmFile = new TFile("mcfm_merged.root");
  mcfmFile->ls();
  TTree* mcfmNtuple = dynamic_cast<TTree*>(mcfmFile->Get("h300"));
  mcfmNtuple->Print();
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  // Create some histograms to be filled :
  // =====================================
  TH1F* im34    = new TH1F("im34","im34",200,0.,200.);  
  TH1F* im56    = new TH1F("im56","im56",200,0.,200.);  
  TH1F* pt34    = new TH1F("pt34","pt34",200,0.,200.);  
  TH1F* pt56    = new TH1F("pt56","pt56",200,0.,200.);
  TH1F* im3456  = new TH1F("im3456","im3456",1000,0.,1000.);  
  TH1F* pt3456  = new TH1F("pt3456","pt3456",100,0.,50.);
  // --------------------------------------------------------------------------

  // Loop through the different ntuples, and the contents of each :
  // ==============================================================
  Int_t nEvent = (Int_t)(mcfmNtuple->GetEntries());  
  cout << " Looping over the " << nEvent << " events in the ntuple." << endl;


  // Get pointers to the requisite branches :
  // ========================================
  TBranch* wtBranch   = mcfmNtuple->GetBranch("wt");
  
  TBranch* px3Branch  = mcfmNtuple->GetBranch("px3");
  TBranch* py3Branch  = mcfmNtuple->GetBranch("py3");
  TBranch* pz3Branch  = mcfmNtuple->GetBranch("pz3");
  TBranch* E3Branch   = mcfmNtuple->GetBranch("E3");
  TBranch* px4Branch  = mcfmNtuple->GetBranch("px4");
  TBranch* py4Branch  = mcfmNtuple->GetBranch("py4");
  TBranch* pz4Branch  = mcfmNtuple->GetBranch("pz4");
  TBranch* E4Branch   = mcfmNtuple->GetBranch("E4");
  TBranch* px5Branch  = mcfmNtuple->GetBranch("px5");
  TBranch* py5Branch  = mcfmNtuple->GetBranch("py5");
  TBranch* pz5Branch  = mcfmNtuple->GetBranch("pz5");
  TBranch* E5Branch   = mcfmNtuple->GetBranch("E5");
  TBranch* px6Branch  = mcfmNtuple->GetBranch("px6");
  TBranch* py6Branch  = mcfmNtuple->GetBranch("py6");
  TBranch* pz6Branch  = mcfmNtuple->GetBranch("pz6");
  TBranch* E6Branch   = mcfmNtuple->GetBranch("E6");

  // In general there is no warning issued if you try to read 
  // past the last event in the ntuple, so be careful here :
  for (Int_t iEvent = 0; iEvent < nEvent; iEvent++)
    {
      if (iEvent%100000 == 0) 
      	{
	  cout << " Reading event " << iEvent << endl;
	}
      
      mcfmNtuple->GetEvent(iEvent);

      float wt = wtBranch->GetLeaf("wt")->GetValue();
      // cout << " eventweight = " << wt << endl;

      // Form the relevant four-vectors :
      // ================================
      float px3 = px3Branch->GetLeaf("px3")->GetValue(0);
      float py3 = py3Branch->GetLeaf("py3")->GetValue(0);
      float pz3 = pz3Branch->GetLeaf("pz3")->GetValue(0);
      float E3  = E3Branch->GetLeaf("E3")->GetValue(0);
      TLorentzVector p3(px3,py3,pz3,E3);

      float px4 = px4Branch->GetLeaf("px4")->GetValue(0);
      float py4 = py4Branch->GetLeaf("py4")->GetValue(0);
      float pz4 = pz4Branch->GetLeaf("pz4")->GetValue(0);
      float E4  = E4Branch->GetLeaf("E4")->GetValue(0);
      TLorentzVector p4(px4,py4,pz4,E4);

      float px5 = px5Branch->GetLeaf("px5")->GetValue(0);
      float py5 = py5Branch->GetLeaf("py5")->GetValue(0);
      float pz5 = pz5Branch->GetLeaf("pz5")->GetValue(0);
      float E5  = E5Branch->GetLeaf("E5")->GetValue(0);
      TLorentzVector p5(px5,py5,pz5,E5);

      float px6 = px6Branch->GetLeaf("px6")->GetValue(0);
      float py6 = py6Branch->GetLeaf("py6")->GetValue(0);
      float pz6 = pz6Branch->GetLeaf("pz6")->GetValue(0);
      float E6  = E6Branch->GetLeaf("E6")->GetValue(0);
      TLorentzVector p6(px6,py6,pz6,E6);

      float mass34 = (p3+p4).M();
      float mass56 = (p5+p6).M();
      float transmom34 = (p3+p4).Perp();
      float transmom56 = (p5+p6).Perp();
      float mass3456 = (p3+p4+p5+p6).M();
      float transmom3456 = (p3+p4+p5+p6).Perp();
      

      // Apply cuts :
      if (mass56 > 0.0)
	{

	  // Fill the actual histograms :
	  // ============================
	  im34->Fill(mass34,wt);
	  im56->Fill(mass56,wt);
	  pt34->Fill(transmom34,wt);
	  pt56->Fill(transmom56,wt);
	  im3456->Fill(mass3456,wt);
	  pt3456->Fill(transmom3456,wt);

	} // cuts

    }

  // From now on, read/write to the output ROOT file :
  // =================================================
  hfile->cd();

  // Plot the histograms :
  // =====================
  padPointers[0]->cd();
  padPointers[0]->SetLogy();
  // cout << " im34 maximum = " << im34->GetBinContent(im34->GetMaximumBin()) << endl;
  // cout << " im34 minimum = " << im34->GetBinContent(im34->GetMinimumBin()) << endl;
  im34->SetMinimum(0.0001);
  im34->SetMaximum(100.0);
  im34->SetLineWidth(4);
  im34->Draw();
  im34->Write();

  padPointers[1]->cd();
  // padPointers[1]->SetLogy();
  pt34->SetLineWidth(4);
  pt34->Draw();
  pt34->Write();

  padPointers[2]->cd();
  padPointers[2]->SetLogy();
  im56->SetMinimum(0.0001);
  im56->SetMaximum(100.0);
  im56->SetLineWidth(4);
  im56->Draw();
  im56->Write();

  padPointers[3]->cd();
  // padPointers[3]->SetLogy();
  pt56->SetLineWidth(4);
  pt56->Draw();
  pt56->Write();

  padPointers[4]->cd();
  padPointers[4]->SetLogy();
  im3456->SetMinimum(0.0001);
  im3456->SetMaximum(100.0);
  im3456->SetLineWidth(4);
  im3456->Draw();
  im3456->Write();

  padPointers[5]->cd();
  // padPointers[5]->SetLogy();
  pt3456->SetLineWidth(4);
  pt3456->Draw();
  pt3456->Write();

  c1->Update();
  ps->Close();

  // Close the output ROOT file :
  // ============================
  hfile->Close();

  return 0;
  
}
